@@ -14,9 +14,10 @@
 # limitations under the License.
 #
 
-from ._base import Document, JDocument, JQuery, get_topics
+from ._base import Document, JDocument, JQuery, get_topics, get_topics_with_reader
 from ._searcher import JSimpleSearcherResult, LuceneSimilarities, SimpleFusionSearcher, SimpleSearcher
 from ._nearest_neighbor import SimpleNearestNeighborSearcher, JSimpleNearestNeighborSearcherResult
 
 __all__ = ['Document', 'JDocument', 'JQuery', 'LuceneSimilarities', 'SimpleFusionSearcher', 'SimpleSearcher',
-           'JSimpleSearcherResult', 'SimpleNearestNeighborSearcher', 'JSimpleNearestNeighborSearcherResult', 'get_topics']
+           'JSimpleSearcherResult', 'SimpleNearestNeighborSearcher', 'JSimpleNearestNeighborSearcherResult',
+           'get_topics', 'get_topics_with_reader']
