@@ -18,10 +18,6 @@ ZOOKEEPER_URL=$2
 # Copy anserini into lib dir
 mkdir ${SOLR_DIR}/lib && cp ../../../../target/anserini-*-fatjar.jar ${SOLR_DIR}/lib
 
-# Extract lang.zip in each configset
-unzip anserini/conf/lang.zip -d anserini/conf
-unzip anserini-twitter/conf/lang.zip -d anserini-twitter/conf
-
 # Upload configset to Solr
 ${SOLR_DIR}/bin/solr zk -z ${ZOOKEEPER_URL:-localhost:9983} upconfig -n anserini -d anserini
 ${SOLR_DIR}/bin/solr zk -z ${ZOOKEEPER_URL:-localhost:9983} upconfig -n anserini-twitter -d anserini-twitter
